@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2014 Bradley Spengler, Open Source Security, Inc.
+ * Copyright (C) 2002-2016 Bradley Spengler, Open Source Security, Inc.
  *        http://www.grsecurity.net spender@grsecurity.net
  *
  * This file is part of gradm.
@@ -296,6 +296,28 @@ check_default_objects(struct role_acl *role)
 	return;
 }
 
+static unsigned int
+check_nested_default_objects(void)
+{
+	struct proc_acl *tmp;
+	struct file_acl *tmpf;
+	unsigned int errs_found = 0;
+
+	for_each_nested_subject(tmp) {
+		/* skip all inherited subjects */
+		if (tmp->parent_subject != NULL)
+			continue;
+		tmpf = lookup_acl_object_by_name(tmp, "/");
+		if (tmpf == NULL) {
+			fprintf(stderr, "Default object not found for "
+				"nested subject %s\n", tmp->filename);
+			errs_found++;
+		}
+	}
+
+	return errs_found;
+}
+
 static void
 check_subject_modes(struct role_acl *role)
 {
@@ -681,6 +703,8 @@ analyze_acls(void)
 
 	errs_found += check_role_transitions();
 
+	errs_found += check_nested_default_objects();
+
 	for_each_role(role, current_role)
 		if (role->roletype & GR_ROLE_DEFAULT)
 			def_role_found = 1;
@@ -787,7 +811,7 @@ analyze_acls(void)
 
 		chk.u_modes = GR_WRITE;
 
-		if (!check_permission(role, def_acl, "/boot", &chk)) {
+		if (!stat("/boot", &fstat) && !check_permission(role, def_acl, "/boot", &chk)) {
 			fprintf(stderr,
 				"Writing access is allowed by role %s to /boot, the directory which "
 				"holds boot and kernel information.\n\n",
@@ -795,7 +819,7 @@ analyze_acls(void)
 			errs_found++;
 		}
 
-		if (!check_permission(role, def_acl, "/run", &chk)) {
+		if (!stat("/run", &fstat) && !check_permission(role, def_acl, "/run", &chk)) {
 			fprintf(stderr,
 				"Writing access is allowed by role %s to /run, the directory which "
 				"holds information for running services and potentially the initctl device.\n\n",
@@ -973,7 +997,7 @@ analyze_acls(void)
 			errs_found++;
 		}
 
-		if (!check_permission(role, def_acl, "/boot", &chk)) {
+		if (!stat("/boot", &fstat) && !check_permission(role, def_acl, "/boot", &chk)) {
 			fprintf(stderr,
 				"Reading access is allowed by role %s to "
 				"/boot, the directory which holds kernel "
